using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace M_C_V
{    
    /// <summary>
    /// Project demonstrates grafting of m-C-v on
    /// M-VC C# Windows Form Program
    /// 1) Modified our Mortgage Calculator to use immutable structures for thread safety
    /// 2) Abstracts interaction between Model and Controller,
    /// 3) Abstracts interaction between Controller and View
    /// allowing substitution/reuse of Model and View code
    /// 
    /// All interactions between View and Model go through the Controller
    /// so Model and View have no implementation knowledge of each other.
    /// Model implements Mortgage.ICalculate
    /// Controller interacts with the Model via ICalculate
    /// 
    /// Views registers for events with the Controller
    /// Controller fires NotifyHandler events when Model state changes
    /// Controller implements Mortgage.IUpdate
    /// Form/System calls Controller.IUpdate as needed to request changes in Model
    /// 
    /// Communication between "levels" is done using structures
    /// Form1 passes MortgageParameter struct to Controller via IUpdate
    /// Controller passes MortgageEventArg to View via NotifyHandler event
    /// MortgageEventArg in turn, contains a MortgageCalculation struct
    /// Demonstrates use of adapter and separate View class MortgageTotalView
    /// 
    /// Updated 11.18.06 Added MCI.IViewable interface, MCV.IViewableEvents interface,
    /// a second View (FormView) and two controllers controllerAdmin and controllerUser.
    /// 
    /// Updated 11.20.06 Added Support for MCV.IVievable.Reset() and MCV.IVievableEvents.Reset()
    /// Added controllerAllViews to support only resetting and clearing all Views
    /// thus allowing us to clear and reset all Views when user switches log in privileges
    /// or when user enables/disables Calculation and Total.
    /// 
    /// Updated 11.21.06 to use AbstractMCVView as a base class. MortgageTotalDerivedView and FormDerivedView
    /// inherit from the AbstractMCVView class.
    ///  </summary>
    public partial class Form1 : Form
    {
        Controller controllerAdmin = new Controller();
        Controller controllerUser = new Controller();
        Controller controllerCurrent; // active controller
        Controller controllerAllViews = new Controller();

        //FormView formView;
        FormDerivedView formView;
        //MortgageTotalView totalView;
        MortgageTotalDerivedView totalView;

        private bool isAdministrator = true;
        private string disableMessage = "Disabled";

        public Form1()
        {
            InitializeComponent();
            if (isAdministrator)
            {
                controllerCurrent = controllerAdmin;
            }
            else
            {
                controllerCurrent = controllerUser;
            }

            // instantiate Views
            //formView = new FormView(textBoxPrincipal,
            formView= new FormDerivedView(textBoxPrincipal,
                                       textBoxInterest,
                                       textBoxMonths,
                                       textBoxPayment,
                                       textBoxMessage);
            //totalView = new MortgageTotalView(textBoxTotal);
            totalView = new MortgageTotalDerivedView(textBoxTotal);

            // FORM1 ALL VIEWS CONTROLLER
            // register ALL VIEWS but only with Clear and Reset handlers only
            // WILL THROW null reference exception on attempt to fire Disable, Enable or Refresh
            // FORM1 has "no permission" to disable, enable or refresh all views
            // Use currentController to enable, refresh or start calculation
            controllerAllViews.ClearDispatcher += formView.GetClearHandler();
            controllerAllViews.ResetDispatcher += formView.GetResetHandler();
            controllerAllViews.ClearDispatcher += totalView.GetClearHandler();
            controllerAllViews.ResetDispatcher += totalView.GetResetHandler();


            // ADMINISTRATOR CONTROLLER
            // register FormView component with event dispatchers
            controllerAdmin.NotifyDispatcher += formView.GetNotifyHandler();
            controllerAdmin.DisableDispatcher += formView.GetDisableHandler();
            controllerAdmin.EnableDispatcher += formView.GetEnableHandler();
            controllerAdmin.ClearDispatcher += formView.GetClearHandler();
            controllerAdmin.RefreshDispatcher += formView.GetRefreshHandler();
            controllerAdmin.ResetDispatcher += formView.GetResetHandler();

            // register MortgageTotalView View component with event dispatchers
            controllerAdmin.NotifyDispatcher += totalView.GetNotifyHandler();
            controllerAdmin.DisableDispatcher += totalView.GetDisableHandler();
            controllerAdmin.EnableDispatcher += totalView.GetEnableHandler();
            controllerAdmin.ClearDispatcher += totalView.GetClearHandler();
            controllerAdmin.RefreshDispatcher += totalView.GetRefreshHandler();
            controllerAdmin.ResetDispatcher += totalView.GetResetHandler();

            // USER CONTROLLER limit access to some views here
            // register FormView component with event dispatchers
            controllerUser.NotifyDispatcher += formView.GetNotifyHandler();
            controllerUser.DisableDispatcher += formView.GetDisableHandler();
            controllerUser.EnableDispatcher += formView.GetEnableHandler();
            controllerUser.ClearDispatcher += formView.GetClearHandler();
            controllerUser.RefreshDispatcher += formView.GetRefreshHandler();
            controllerUser.ResetDispatcher += formView.GetResetHandler();
        }

        private void buttonClear_Click(object sender, EventArgs e)
        {
            controllerCurrent.Clear();
        }

        private void buttonCalc_Click(object sender, EventArgs e)
        {
            // FormView will validate user input
            // ParametersBoolStruct wraps user input as
            // Mortage.Parameters struct
            // IsSuccess = true if preconditions are validated, 
            // else IsSuccess = false
            Mortgage.ParametersBoolStruct boolStruct = formView.GetParameters();
            if (boolStruct.IsSuccess && (boolStruct.Parameters.Target != Mortgage.INVALID))
            {
                controllerCurrent.Update(boolStruct.Parameters);
            }
        }

        private void aboutMCalcToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Twisted Mortgage Calculator", "JAL Computing",
     MessageBoxButtons.OK);
        }

        private void enableCalculationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            enableCalculationToolStripMenuItem.Checked = true;
            disableCalculationToolStripMenuItem.Checked = false;
            buttonCalc.Enabled = true;
            controllerCurrent.Enable();
        }

        private void disableCalculationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            enableCalculationToolStripMenuItem.Checked = false;
            disableCalculationToolStripMenuItem.Checked = true;
            buttonCalc.Enabled = false;
            controllerCurrent.Disable(this.disableMessage);
        }

        private void enableTotalToolStripMenuItem_Click(object sender, EventArgs e)
        {
            disableTotalToolStripMenuItem.Checked = false;
            enableTotalToolStripMenuItem.Checked = true;
            totalView.Enable();
        }

        private void disableTotalToolStripMenuItem_Click(object sender, EventArgs e)
        {
            disableTotalToolStripMenuItem.Checked = true;
            enableTotalToolStripMenuItem.Checked = false;
            totalView.Clear();
            totalView.Disable(disableMessage);
        }

        private void adminPrivilegesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            adminPrivilegesToolStripMenuItem.Checked = true;
            userPrivilegesToolStripMenuItem.Checked = false;
            isAdministrator = true;
            //controllerCurrent.Clear();
            controllerAllViews.Reset();
            controllerAllViews.Clear();
            controllerCurrent = controllerAdmin;
        }

        private void userPrivilegesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            adminPrivilegesToolStripMenuItem.Checked = false;
            userPrivilegesToolStripMenuItem.Checked = true;
            isAdministrator = false;
            //controllerCurrent.Clear();
            controllerAllViews.Reset();
            controllerAllViews.Clear();
            controllerCurrent = controllerUser;
        }

        private void refreshViewsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            controllerCurrent.Refresh();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
            Application.Exit();
        }
    }
}