
using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace M_C_V
{
    // VIEW CLASS to separate and delineate "domain of responsibility"

    /// <summary>
    /// FORM VIEW Class
    /// class that displays parameters and result of calculations in textboxes
    /// implements IViewable interface so it can respond to Clear,Enable,Disable,Refresh
    /// returns delegates from IViewableEvents that correspond to each method in IViewable
    /// this class validates user input
    /// returning a Mortgage.ParametersBoolStruct to client, see GetParameters()
    /// each IViewable must store Calculation state to support Refresh(), see lastCalculation
    /// </summary>
    public class FormView : MCV.IViewable, MCV.IViewableEvents
    {
        private TextBox textBoxPrincipal;
        private TextBox textBoxInterest;
        private TextBox textBoxMonths;
        private TextBox textBoxPayment;
        private TextBox textBoxMessage;
        private bool isEnabled = true;
        private string disableMessage = "";
        private Mortgage.Calculation defaultCalculation =
                        new Mortgage.Calculation(0, 0, 0, 0, Mortgage.INVALID, "");
        private Mortgage.Calculation lastCalculation;



        // Constructor
        // Any control passed in here is in domain of responsibility of this class
        // this class is responsible for properly updating state of contained controls
        // in response to controller events
        // note buttonCalculate is NOT in domain of responsibility of this class
        public FormView(TextBox textBoxPrincipal,
                                TextBox textBoxInterest,
                                TextBox textBoxMonths,
                                TextBox textBoxPayment,
                                TextBox textBoxMessage)
        {
            this.textBoxPrincipal = textBoxPrincipal;
            this.textBoxInterest = textBoxInterest;
            this.textBoxMonths = textBoxMonths;
            this.textBoxPayment = textBoxPayment;
            this.textBoxMessage = textBoxMessage;
            this.lastCalculation = defaultCalculation;

            this.textBoxPayment.Text = "0";
        }

        public Mortgage.Calculation Calculation
        {
            get { return this.lastCalculation; }
        }

        // this class is responsible for validating
        // preconditions and communicating with client via ParametersBoolStruct
        public Mortgage.ParametersBoolStruct GetParameters()
        {
            double principal = 0;
            double interest = 0;
            int months = 0;
            double payment = 0;
            bool isInputError = false;
            int target = Mortgage.INVALID;

            Mortgage.Parameters invalidParameters = new Mortgage.Parameters(principal,
                                                            interest,
                                                            months,
                                                            payment,
                                                            Mortgage.INVALID);
            Mortgage.ParametersBoolStruct invalidStruct = new Mortgage.ParametersBoolStruct(false,
                                                            "Invalid Input",
                                                            "FormView",
                                                            invalidParameters);

            double[] arrayDb = new double[4];

            // validate user input, must allow zero
            try
            {
                principal = Double.Parse(textBoxPrincipal.Text);
                if (principal < 0)
                {
                    throw new Exception();
                }
                arrayDb[Mortgage.PRINCIPAL] = principal;
            }
            catch
            {
                textBoxPrincipal.Text = "Invalid Input.";
                isInputError = true;
            }
            try
            {
                interest = Double.Parse(textBoxInterest.Text);
                if ((interest < 0) || (interest > 100))
                {
                    throw new Exception();
                }
                arrayDb[Mortgage.INTEREST] = interest;
            }
            catch
            {
                textBoxInterest.Text = "Invalid Input.";
                isInputError = true;
            }
            try
            {
                months = Int32.Parse(textBoxMonths.Text);
                if (months < 0)
                {
                    throw new Exception();
                }
                arrayDb[Mortgage.MONTHS] = months;
            }
            catch
            {
                textBoxMonths.Text = "Invalid Input.";
                isInputError = true;
            }
            try
            {
                payment = Double.Parse(textBoxPayment.Text);
                if (payment < 0)
                {
                    throw new Exception();
                }
                arrayDb[Mortgage.PAYMENT] = payment;
            }
            catch
            {
                textBoxPayment.Text = "Invalid Input.";
                isInputError = true;
            }
            if (isInputError)
            {
                return invalidStruct;
            }
            // one, and only one, "value" must be zero --> target
            int zeros = 0;
            for (int index = 0; index < 4; index++)
            {
                if (arrayDb[index] == 0)
                {
                    zeros++;
                    target = index;
                }
            }
            if (zeros > 1)
            {
                textBoxMessage.Text = "Too many zero parameters.";
                isInputError = true;
                return invalidStruct;
            }
            if (zeros == 0)
            {
                textBoxMessage.Text = "One value must be zero.";
                isInputError = true;
                return invalidStruct;
            }
            // success
            Mortgage.Parameters parameters = new Mortgage.Parameters(principal,
                                                       interest,
                                                       months,
                                                       payment,
                                                       target);
            return new Mortgage.ParametersBoolStruct(true, "", "FormView", parameters);
        }

        // implement IViewable methods
        public bool Clear()
        {
            textBoxMessage.Text = "";
            ClearControls();
            return true;
        }
        // restore data from LOCAL state
        public bool Refresh()
        {
            if (lastCalculation.IsValid())
            {
                RefreshView(lastCalculation);
                return true;
            }
            else
            {
                this.Clear();
                return false;
            }
        }
        public bool Enable()
        {
            isEnabled = true;
            return true;
        }
        // reset and clear
        public bool Disable(string message)
        {
            isEnabled = false;
            if (message != null)
            {
                disableMessage = message;
            }
            else
            {
                disableMessage = "";
            }
            this.Reset();
            this.Clear();
            return true;
        }
        public bool Reset()
        {
            this.lastCalculation = defaultCalculation;
            return true;
        }
        // helper method for Clear, Refresh
        private void ClearControls()
        {
            textBoxPrincipal.Text = "";
            textBoxInterest.Text = "";
            textBoxMonths.Text = "";
            textBoxPayment.Text = "0";
            textBoxPrincipal.Focus();
        }
        // private helper method for Refresh
        // works with data pushed from controller
        // works with data stored in this view
        private void RefreshView(Mortgage.Calculation mortgage)
        {
            // enable/disable logic
            if (isEnabled == false)
            {
                textBoxMessage.Text = disableMessage;
                ClearControls();
                return;
            }
            // struct cannot be null!
            if (mortgage.IsValid())
            {
                lastCalculation = mortgage;
                textBoxPrincipal.Text = mortgage.Principal.ToString();
                textBoxInterest.Text = mortgage.Interest.ToString();
                textBoxMonths.Text = mortgage.Months.ToString();
                textBoxPayment.Text = mortgage.Payment.ToString();
                textBoxMessage.Text = mortgage.Message.ToString();
                return;
            }
            else
            {
                textBoxMessage.Text = mortgage.Message.ToString();
                ClearControls();
                return;
            }
        }

        // INotifyHandler
        // data is pushed from Controller
        public void FormView_OnNotifyEvent(object source, Mortgage.MortgageEventArgs e)
        {
            if (isEnabled)
            {
                Mortgage.Calculation calculation = e.calculation;
                RefreshView(calculation);
            }
            else
            {
                textBoxMessage.Text = disableMessage;
                ClearControls();
            }
        }
        public Mortgage.NotifyHandler GetNotifyHandler()
        {
            return new Mortgage.NotifyHandler(FormView_OnNotifyEvent);
        }

        
        // Provide methods that return delegates for IViewableEvent handlers
        public MCV.DisableHandler GetDisableHandler()
        {
            return new MCV.DisableHandler(FormView_OnDisableEvent);
        }
        public MCV.EnableHandler GetEnableHandler()
        {
            return new MCV.EnableHandler(FormView_OnEnableEvent);
        }
        public MCV.ClearHandler GetClearHandler()
        {
            return new MCV.ClearHandler(FormView_OnClearEvent);
        }
        public MCV.RefreshHandler GetRefreshHandler()
        {
            return new MCV.RefreshHandler(FormView_OnRefreshEvent);

        }
        public MCV.ResetHandler GetResetHandler()
        {
            return new MCV.ResetHandler(FormView_OnResetEvent);
        }

        // Handle IViewableEvents
        public void FormView_OnDisableEvent(object source, MCV.DisableEventArgs e)
        {
            this.Disable(e.Message);
        }
        public void FormView_OnEnableEvent(object source, EventArgs e)
        {
            this.Enable();
        }
        public void FormView_OnClearEvent(object source, EventArgs e)
        {
            this.Clear();
        }
        public void FormView_OnRefreshEvent(object source, EventArgs e)
        {
            // restore from LOCAL data
            this.Refresh();
        }
        public void FormView_OnResetEvent(object source, EventArgs e)
        {
            // reset LOCAL data and clear controls
            this.Reset();
        }
    } // end FormView class
}

