using System;
using System.Collections.Generic;
using System.Text;

namespace M_C_V
{
    // class used to abstract Mortgage algorithm (Model)
    // and commumication between View and Controller
    // not meant to be instantiated
    public abstract class Mortgage : Object
    {
        // static class constants, not "versionable"
        // must recompile to update
        public const int INVALID = -1;  // flags validation error
        public const int PRINCIPAL = 0;
        public const int INTEREST = 1;
        public const int MONTHS = 2;
        public const int PAYMENT = 3;

        public delegate void NotifyHandler(object source, MortgageEventArgs args);

        // Form/System request changes to Model via controller.Update
        // Controller should implement IUpdate
        public interface IUpdate
        {
            bool Update(Mortgage.Parameters mp);
        }

        // Controller could Notify View of change in Model state
        // via INotify.Notify
        // This interface is not used in this example as all Notification is
        // done via events between Controller and interested Views
        public interface INotify
        {
            bool Notify(Mortgage.Calculation mc);
        }

        // Model does calculation based on Parameter struct and
        // returns Calculation struct
        public interface ICalculate
        {
            Mortgage.Calculation Calculate(Mortgage.Parameters mp);
        }

        // wraps bool result, message and source as value type
        // helper properties for bool result: B
        // immutable structure
        // use to return Mortgage.Parameters wrapped in boolean struct
        // if pre-conditions are validated IsSuccess is set to true, else false
        public struct ParametersBoolStruct
        {
            public readonly bool IsSuccess;
            public readonly String Message;
            public readonly String Source;
            public readonly Parameters Parameters;

            public ParametersBoolStruct(bool isSuccess, String message, String source, Parameters parameters)
            {
                if (message == null) { message = ""; }
                if (source == null) { source = ""; }
                this.IsSuccess = isSuccess;
                this.Message = message;
                this.Source = source;
                this.Parameters = parameters;
            }
            public bool B // helper
            {
                get { return IsSuccess; }
            }
        }

        /// <summary>
        /// CALCULATION struct
        /// Immutable value type used to pass data between
        /// Model and Views via Controller 
        /// Should be thread safe
        /// Struct DOES NOT validate data, only checks for valid target
        /// Returns 0 values on invalid target
        /// IsValid was updated 11.15.06 to validate for no arg construction
        /// </summary>
        public struct Calculation
        {
            private double[] arrayDb;
            private int target;
            private string message;

            // populates data structure, does not validate data
            public Calculation(double principal, double interest, int months, double payment, int target, string message)
            {
                arrayDb = new double[4];

                // reset flags
                this.target = target;
                this.message = message;

                arrayDb[PRINCIPAL] = principal;
                arrayDb[INTEREST] = interest;
                arrayDb[MONTHS] = months;
                arrayDb[PAYMENT] = payment;
            }


            // default target is -1 (INVALID)
            // must validate for no arg construction
            public bool IsValid()
            {
                double total = arrayDb[0] + arrayDb[1] + arrayDb[2] + arrayDb[3];
                if (total == 0) { return false; } // no arg construction
                return ((target >= PRINCIPAL) && (target <= PAYMENT)) ? true : false;
            }
            public double Result
            {
                get
                {
                    return IsValid() ? arrayDb[target] : 0.0;
                }
            }
            public int Target
            {
                get
                {
                    return target;
                }
            }
            public string Message
            {
                get
                {
                    return message;
                }
            }
            public double Principal
            {
                get
                {
                    return IsValid() ? arrayDb[PRINCIPAL] : 0.0;
                }
            }
            public double Interest
            {
                get
                {
                    return IsValid() ? arrayDb[INTEREST] : 0.0;
                }
            }
            public int Months
            {
                get
                {
                    return IsValid() ? (int)arrayDb[MONTHS] : 0;
                }
            }
            public double Payment
            {
                get
                {
                    return IsValid() ? arrayDb[PAYMENT] : 0.0;
                }
            }
        }  // end Calculation Struct

        /// <summary>
        /// PARAMETERS Struct
        /// Immutable struct passed from Controller to Model
        /// and between View/Systmem and Controller
        /// Should be thread safe
        /// </summary>
        public struct Parameters
        {
            public readonly double Principal;
            public readonly double Interest;
            public readonly int Months;
            public readonly double Payment;
            public readonly int Target;



            public Parameters(double principal, double interest, int months, double payment, int target)
            {
                Principal = principal;
                Interest = interest;
                Months = months;
                Payment = payment;
                Target = Mortgage.INVALID;
                if ((target >= PRINCIPAL) && (target <= PAYMENT))
                {
                    Target = target;
                }
            }
        }

        public class MortgageEventArgs : EventArgs
        {
            public readonly Mortgage.Calculation calculation;
            public MortgageEventArgs(Mortgage.Calculation calculation)
            {
                this.calculation = calculation;
            }
        }
    } // end class Mortgage
}
