using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace M_C_V
{
    // VIEW CLASS to separate and delineate "domain of responsibility"

    /// <summary>
    /// MORTGAGE TOTAL Class
    /// class that calculate total mortgage and displays result in a textbox
    /// implements IViewable interface so it can respond to Clear,Enable,Disable,Refresh
    /// returns delegates from IViewableEvents that correspond to each method in IViewable
    /// each View must store local state for Refresh()
    /// </summary>
    public class MortgageTotalView : MCV.IViewable, MCV.IViewableEvents
    {
        private TextBox tb;     
        private Mortgage.Calculation defaultCalculation= new Mortgage.Calculation(0, 0, 0, 0, Mortgage.INVALID, "");
        private Mortgage.Calculation lastCalculation;

        private bool isEnabled = true;
        private string disableMessage = "";

        public Mortgage.Calculation Calculation
        {
            get { return this.lastCalculation; }
        }

        // Constructor
        // ASSERT tb != null
        public MortgageTotalView(TextBox tb) 
        {
            if (tb == null)
            {
                throw new ArgumentNullException();
            }
            this.tb = tb;
            this.lastCalculation = defaultCalculation;
        }

        // implement IViewable methods
        public bool Clear()
        {
            tb.Clear();
            return true;
        }
        // refresh from LOCAL data
        public bool Refresh()
        {
            if (lastCalculation.IsValid())
            {
                RefreshView(lastCalculation);
                return true;
            }
            else
            {
                this.Clear();
                return false;
            }
        }
        public bool Enable()
        {
            isEnabled = true;
            return true;
        }
        // reset and clear
        public bool Disable(string message)
        {
            isEnabled = false;
            if (message != null)
            {
                disableMessage = message;
            }
            else
            {
                disableMessage = "";
            }
            this.Reset();
            this.Clear();
            return true;
        }
        public bool Reset()
        {
            this.lastCalculation = defaultCalculation;
            return true;
        }
        // helper method for Refresh
        private void RefreshView(Mortgage.Calculation mc)
        {
            if (isEnabled)
            {
                if ((tb != null) && mc.IsValid())
                {
                    lastCalculation = mc;  // store last Mortgage.Calculation
                    double total = mc.Payment * mc.Months;
                    tb.Text = String.Format("${0:N}", total);
                }
                else
                {
                    tb.Text = "Error.";
                }
            }
            else
            {
                tb.Text = disableMessage;
            }
        }

        // Provide methods that return delegates for IViewable handlers
        public Mortgage.NotifyHandler GetNotifyHandler()
        {
            return new Mortgage.NotifyHandler(MortgageTotalView_OnNotifyEvent);
        }
        public MCV.DisableHandler GetDisableHandler()
        {
            return new MCV.DisableHandler(MortgageTotalView_OnDisableEvent);
        }
        public MCV.EnableHandler GetEnableHandler()
        {
            return new MCV.EnableHandler(MortgageTotalView_OnEnableEvent);
        }
        public MCV.ClearHandler GetClearHandler()
        {
            return new MCV.ClearHandler(MortgageTotalView_OnClearEvent);
        }
        public MCV.RefreshHandler GetRefreshHandler()
        {
            return new MCV.RefreshHandler(MortgageTotalView_OnRefreshEvent);

        }
        public MCV.ResetHandler GetResetHandler() 
        {
            return new MCV.ResetHandler(MortgageTotalView_OnResetEvent);
        }
        // Actual event handlers
        public void MortgageTotalView_OnNotifyEvent(object source, Mortgage.MortgageEventArgs e)
        {
            Mortgage.Calculation c = e.calculation;
            // restore from LOCAL data
            RefreshView(c);
        }
        public void MortgageTotalView_OnDisableEvent(object source, MCV.DisableEventArgs e)
        {
            this.Disable(e.Message);
        }
        public void MortgageTotalView_OnEnableEvent(object source, EventArgs e)
        {
            this.Enable();
        }
        public void MortgageTotalView_OnClearEvent(object source, EventArgs e)
        {
            this.Clear();
        }
        public void MortgageTotalView_OnRefreshEvent(object source, EventArgs e)
        {
            this.Refresh();
        }
        public void MortgageTotalView_OnResetEvent(object source, EventArgs e)
        {
            this.Reset();
        }

    } // end MortgageTotalView Class

}

